;********************************************************************************************
;* Title:  PACED_LOOP.ASM                      (c) MOTOROLA Inc. 2003 All rights reserved.
;********************************************************************************************
;* Author: Bill Lucas - Motorola TSPG
;*
;* Description: Software paced loop demo
;*
;* This program is a software paced loop framework. It calls user tasks every 100
;* milliseconds. This implementation has three user tasks. The first maintanis a
;* 0-60 second software clock. The second user task blinks LED1 every second. The
;* third user task blinks LED4 every 4 seconds.
;*
;* Documentation: 9S08GB60 family Data Sheet and HCS08 Family Reference Manual (HCS08RM1/D)
;*
;* Include Files: None
;*
;* Assembler:  Metrowerks Code Warrior 3.0 (pre-release)
;*             or P&E Microcomputer Systems - CASMS08 (beta v4.02)
;*
;* Revision History:  not yet released
;* Rev #     Date       Who     Comments
;* -----  -----------  ------  --------------------------------------------
;*  1.2    25-Apr-03   B.Lucas  Correct minor typos in comments
;*  1.1    15-Apr-03   B.Lucas  Comments and modify for CW 3.0 project
;*  1.0    02-Apr-03   B.Lucas  Initial code release
;********************************************************************************************
             base  10t         ;change default to decimal

PACED_LOOP   equ           *
;
;* Set-up the timer to interrupt every 100ms on overflow
             ldhx   #TPM_COUNT    ;set mod = 14745
             sthx   TPM1MODH      ;overflow every 100 ms. 
;                                 = 18.874e6/128/14745
             mov    #(mTOIE|mCLKSA|mPS2|mPS1|mPS0),TPM1SC
;                                 ov int en,bus/128
;
             cli
;* Set-up I/O ports. We will use LED's 3 and 4.
             mov    #(mPTFDD3|mPTFDD0),PTFD ;Port D bits 0:3 = 1
             mov    #(mPTFDD3|mPTFDD0),PTFDD ;port D bits 0:3 outputs
;
;* Initialize variables
             clr    Tick_100ms    ;initial value = 0
             clr    Tock_1sec     ;initial value = 0
;
;* MAIN - Beging of main program loop
;* Loop is executed once every 100ms. A pass through all major task 
;* routines takes less than lOOmS and then time is wasted until the
;* MSB of Tick_100ms is set (every 100ms). At each timer overflo
;* interrupt,the overflow flag is cleared. 
;
;* The variable Tick_100ms (lower 4 bits) keep track of lOOmS
;* periods. When the lower four bits of Tick_100ms increments from
;* 9 to 10 it is cleared to 0 and Tock_1sec is incremented.	
;
MAIN         equ    *
             brclr  MAIN_flag,Tick_100ms,MAIN ;until MAIN_flag set
             lda    Tick_100ms
             and    #FLAG_mask    ;clears Tick_100ms & saves bits 0:3
             inca
             sta    Tick_100ms
             cmp    #ONE_SECOND
             bne    go_user
             clr    Tick_100ms
go_user      equ    *             ;user tasks follow:
             jsr    TIME          ;user time keeping task
             jsr    blink_LED1    ;user blink LED1 task
             jsr    blink_LED4    ;user blink LED4 task
;
;* Other user tasks would go here
             bra    MAIN          ;back to the top for next ov
;
* TIME - Update TOCs
* If	TIC = 0, increment 0->59
* If	TIC not = 0, just skip whole routine
;
TIME         EQU    *             ;Update TOCs
             TST    Tick_100ms    ;Check for Tick_100ms=zero
             BNE    XTIME         ;If not; just exit
             INC    Tock_1sec     ;Tock_1sec=Tock_1sec+1
             LDA    #60	
             CMP    Tock_1sec     ;Did Tock_1sec -> 60 ?
             BNE    XTIME         ;If not; just exit
             CLR    Tock_1sec     ;Tock_1sec's rollover
XTIME        RTS                  ;Return from TIME
;
blink_LED1   EQU    *             ;update LED1
             LDA    Tock_1sec     ;If even, LSB will be zero
             LSRA		      ;Shift LSB to carry
             BCS    LED1OFF	      ;If not, turn off LED1
             BSET   LED1,PTFD     ;Turn on LED1
             BRA    XBLINK1       ;Then exit
LED1OFF      BCLR   LED1,PTFD     ;Turn off LED1
XBLINK1      RTS                  ;Return from BLINK LED1
;
blink_LED4   EQU	*            ;update LED4
             LDA    Tock_1sec     ;If even, LSB will be zero
             LSRA                 ;Shift bit 1 to bit 0
             LSRA                 ;Shift LSB to carry
             BCC    LED4OFF       ;If not, turn off LED4
             BSET   LED4,PTFD     ;Turn on LED4
             BRA    XBLINK4       ;Then exit
LED4OFF      BCLR   LED4,PTFD     ;Turn off LED4
XBLINK4      RTS                  ;Return from BLINK LED4
;
;* Timer overflow interrupt routine
TPM1_OV      equ    *
             lda    TPM1SC        ;clear the overflow flag
             bclr   TOF,TPM1SC    ;second half clearing overflow flag
             bset   7,Tick_100ms  ;set msb for main to run user code
             rti
	